"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const childProcess = __importStar(require("child_process"));
const debug_1 = __importDefault(require("debug"));
const extend_1 = __importDefault(require("extend"));
const merge_stream_1 = __importDefault(require("merge-stream"));
const string_argv_1 = __importDefault(require("string-argv"));
const text_stream_search_1 = __importDefault(require("text-stream-search"));
const debug = debug_1.default('observable-process');
// Runs the given command as a separate, parallel process
// and allows to observe it and interact with it.
class ObservableProcess {
    constructor(args) {
        this.verbose = args.verbose || false;
        this.cwd = args.cwd != null ? args.cwd : process.cwd();
        this.stdout = args.stdout || process.stdout;
        this.stderr = args.stderr || process.stderr;
        this.ended = false;
        this.endedListeners = [];
        this.env = args.env || {};
        this.exitCode = -1;
        this.killed = false;
        const [runnable, params] = this.determineRunnable(args.command, args.commands);
        debug(`starting '${runnable}' with arguments [${params.join(',')}]`);
        this.process = childProcess.spawn(runnable, params, this.spawnOptions());
        this.process.on('close', this._onClose.bind(this));
        this.stdin = this.process.stdin;
        this.textStreamSearch = this.createStdOutErrStreamSearch();
        this.forwardStreams();
    }
    // Returns a TextStream instance that listens on both stdout and stderr
    createStdOutErrStreamSearch() {
        return new text_stream_search_1.default(merge_stream_1.default(this.process.stdout, this.process.stderr));
    }
    // Forwards output streams
    forwardStreams() {
        if (this.stdout) {
            this.process.stdout.on('data', data => {
                this.stdout.write(data.toString());
            });
        }
        if (this.stderr) {
            this.process.stderr.on('data', data => {
                this.stderr.write(data.toString());
            });
        }
    }
    // Determines the process name and parameters to run
    determineRunnable(command, commands) {
        let runnable = '';
        let params = [];
        if (command != null) {
            ;
            [runnable, ...params] = this._splitCommand(command);
        }
        if (commands != null) {
            runnable = commands[0];
            params = commands.splice(1);
        }
        return [runnable, params];
    }
    // Returns the options with which the subprocess is going to be spawned
    spawnOptions() {
        const result = {
            env: {},
            cwd: this.cwd
        };
        extend_1.default(result.env, process.env, this.env);
        return result;
    }
    // Enters the given text into the subprocess.
    // Types the ENTER key automatically.
    enter(text) {
        this.stdin.write(`${text}\n`);
    }
    fullOutput() {
        return this.textStreamSearch.fullText();
    }
    kill() {
        debug('killing the process');
        this.killed = true;
        this.process.kill();
    }
    // notifies all registered listeners that this process has ended
    notifyEnded() {
        for (let resolver of this.endedListeners) {
            resolver({ exitCode: this.exitCode, killed: this.killed });
        }
    }
    _onClose(exitCode) {
        debug(`process has ended with code ${exitCode}`);
        this.exitCode = exitCode;
        this.ended = true;
        if (this.verbose) {
            if (this.stderr)
                this.stderr.write('PROCESS ENDED\n');
            if (this.stderr)
                this.stderr.write(`\nEXIT CODE: ${this.exitCode}`);
        }
        this.notifyEnded();
    }
    pid() {
        return this.process ? this.process.pid : -1;
    }
    waitForEnd() {
        return new Promise(resolve => {
            this.endedListeners.push(resolve);
        });
    }
    // Calls the given handler when the given text shows up in the output
    waitForText(text, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.textStreamSearch.waitForText(text, timeout);
        });
    }
    resetOutputStreams() {
        this.textStreamSearch.reset();
    }
    _splitCommand(command) {
        if (Array.isArray(command)) {
            return command;
        }
        else {
            return string_argv_1.default(command);
        }
    }
}
exports.ObservableProcess = ObservableProcess;
